<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 02/03/2016
 * Time: 08:48
 */

namespace HCM\BTL\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use HCM\Util\HCM;
use HCM\Util\Util;
use HCM\Geral\Services\EnviarEmailService;
use HCM\Util\HCMParameters;

class CandidatosServices {

    const STATUS_ANALISE = 2;
	  const ORG_PADRAO = 0;

    protected $entityManager;
    /** @var Environment */
    protected $environment;
    protected $enviarEmailService;

    public function __construct(EntityManager $entityManager, Environment $environment, HCM $hcm, EnviarEmailService $enviarEmailService, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcmParameters   = $hcmParameters;
        $this->hcm = $hcm;
        //$this->processosServices = $processosServices;
        $this->enviarEmailService = $enviarEmailService;
    }

    public function getCandidatosGeral($parameters, $filters) {
        $filterPlaceHolder = Util::createQueryFiltered($filters);
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = self::ORG_PADRAO;

        $filtro = $this->filtroCandidatosGeral($parameters);
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT *
                FROM
                  (SELECT
                    GPE_PESSOA.NRORG,
                    OPERADOR.DSEMAILOPER,
                    TO_CHAR(DES_CURRICULOGERAL.DTULTATUALIZACAO, 'DD/MM/YYYY') || ' - ' || TO_CHAR(TRUNC(SYSDATE) - TRUNC(DES_CURRICULOGERAL.DTULTATUALIZACAO)) || ' dias' AS DTULTATUALIZACAO,
                    GPE_PESSOA.NRPESSOA,
                    GPE_PESSOA.NRPARCNEGOCIO,
                    GPE_PESSOAH.NMPESSOA,
                    GPE_PESSOAH.NRCPFPESSOA,
                    TO_CHAR(GPE_PESSOAH.DTNASCPESSOA, 'DD/MM/YYYY') AS DTNASCPESSOA,
                    FLOOR(TO_NUMBER(SYSDATE - GPE_PESSOAH.DTNASCPESSOA) / 365.25) AS IDADE,
                    DECODE(GPE_PESSOAH.IDSEXOPESSOA, 'M', 'Masculino', 'F', 'Feminino', GPE_PESSOAH.IDSEXOPESSOA) AS IDSEXOPESSOA,
                    GPE_PESSOAH.CDESTACIVIL,
                    ESTADOCIVIL.NMESTACIVIL,
                    GPE_NACIONALIDADE.NRNACIONALIDADE,
                    GPE_PESSOAH.NRGRAUINSTR,
                    GPE_PESSOAH.IDPOSSUIFILHO,
                    GPE_PESSOAH.NRIDADEFILHO,
                    GPE_PESSOAH.NRORGULTATU,
                    GPE_PESSOAH.CDOPERULTATU,
                    GPE_PESSOAH.DTULTATU,
                    GPE_PESSOAH.NRORGINCLUSAO,
                    GPE_PESSOAH.CDOPERINCLUSAO,
                    GPE_PESSOAH.DTINCLUSAO,
                    COMUNICAPARC.CDFORMACOMU,
                    COMUNICAPARC.CDCOMUNICAPARC,
                    COMUNICAPARC.NRCOMUNICAPARC,
                    FORMACOMU.NMFORMACOMU,
                    LOGRADOURO.DSLOGRADOURO || ' ' || INITCAP(ENDERECOPARC.DSENDERECO) AS DSENDERECO,
                    ENDERECOPARC.NRIMOVELENDERECO,
                    ENDERECOPARC.DSCOMPLEENDERECO,
                    ENDERECOPARC.NRCEPENDERECO,
                    ENDERECOPARC.NMBAIRROENDERECO,
                    ENDERECOPARC.CDMUNICIPIO,
                    ENDERECOPARC.SGESTADO,
                    ENDERECOPARC.CDPAIS,
                    PAIS.NMPAIS,
                    INITCAP(MUNICIPIO.NMMUNICIPIO) || ' - ' || ENDERECOPARC.SGESTADO AS LOCALIDADE,
                    MUNICIPIO.NMMUNICIPIO,
                    GPE_GRAUINSTR.NMGRAUINSTR,
                    GPE_NACIONALIDADE.DSNACIONALIDADE,
                    (SELECT COUNT(NRPROCPARTICIPA)
                       FROM DES_PARTICIPAPROCESSO
                      WHERE NRORG             = :P_NRORG
                        AND NRPARCNEGOCIOCAND = GPE_PESSOA.NRPARCNEGOCIO) AS QTD_PROCESSOS,
                    (SELECT CASE
                              WHEN COUNT(1) > 0 THEN 'Sim'
                              ELSE 'Não'
                            END IDFOTO
                       FROM GPE_ANEXOS
                      WHERE NRORG         = GPE_PESSOA.NRORG
                        AND NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                        AND IDFOTO        = 'S') AS IDFOTO
                  FROM GPE_PESSOA
                  LEFT JOIN GPE_PESSOAH
                    ON GPE_PESSOAH.NRPESSOA = GPE_PESSOA.NRPESSOA
                   AND GPE_PESSOAH.NRORG   = GPE_PESSOA.NRORG
                   AND GPE_PESSOAH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                      FROM GPE_PESSOAH H
                                                     WHERE NRORG    = GPE_PESSOA.NRORG
                                                       AND NRPESSOA = GPE_PESSOA.NRPESSOA)
                 INNER JOIN DES_CURRICULOGERAL
                    ON DES_CURRICULOGERAL.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                   AND DES_CURRICULOGERAL.NRORG         = GPE_PESSOA.NRORG                                                       
                  LEFT JOIN COMUNICAPARC
                    ON COMUNICAPARC.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                   AND COMUNICAPARC.NRORG         = GPE_PESSOA.NRORG
                   AND COMUNICAPARC.CDFORMACOMU   = '05'
                  LEFT JOIN ENDERECOPARC
                    ON ENDERECOPARC.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                   AND ENDERECOPARC.NRORG         = GPE_PESSOA.NRORG
                  LEFT JOIN LOGRADOURO
                    ON LOGRADOURO.CDLOGRADOURO = ENDERECOPARC.CDLOGRADOURO
                  LEFT JOIN PAIS
                    ON PAIS.CDPAIS = ENDERECOPARC.CDPAIS
                  LEFT JOIN MUNICIPIO 
                    ON MUNICIPIO.CDMUNICIPIO = ENDERECOPARC.CDMUNICIPIO
                   AND MUNICIPIO.CDPAIS = ENDERECOPARC.CDPAIS
                   AND MUNICIPIO.SGESTADO = ENDERECOPARC.SGESTADO
                  LEFT JOIN PARCNEGOCIO
                    ON PARCNEGOCIO.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                   AND PARCNEGOCIO.NRORG         = GPE_PESSOA.NRORG
                  LEFT JOIN OPERADOR
                    ON OPERADOR.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                   AND OPERADOR.NRORG         = GPE_PESSOA.NRORG
                 INNER JOIN PARCERIA
                    ON PARCERIA.NRPARCNEGOCIO  = GPE_PESSOA.NRPARCNEGOCIO
                   AND PARCERIA.CDTIPOPARCEIRO = 'CANDIDATO'
                   AND PARCERIA.NRORG          = GPE_PESSOA.NRORG                  
                  LEFT JOIN GPE_GRAUINSTR
                    ON GPE_PESSOAH.NRGRAUINSTR = GPE_GRAUINSTR.NRGRAUINSTR
                  LEFT JOIN GPE_NACIONALIDADE
                    ON GPE_PESSOAH.NRNACIONALID = GPE_NACIONALIDADE.NRNACIONALIDADE
                  LEFT JOIN ESTADOCIVIL 
                    ON ESTADOCIVIL.CDESTACIVIL = GPE_PESSOAH.CDESTACIVIL
                  LEFT JOIN FORMACOMU 
                    ON FORMACOMU.CDFORMACOMU = COMUNICAPARC.CDFORMACOMU  
                  ) GPE_PESSOA
            WHERE (GPE_PESSOA.NRCOMUNICAPARC = (SELECT MAX(NRCOMUNICAPARC)
                                                  FROM COMUNICAPARC
                                                 WHERE COMUNICAPARC.NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                                                   AND COMUNICAPARC.NRORG         = GPE_PESSOA.NRORG
                                                   AND COMUNICAPARC.CDFORMACOMU   = '05')
                   OR GPE_PESSOA.NRCOMUNICAPARC   IS NULL)
              AND ((FLOOR(TO_NUMBER(SYSDATE - TO_DATE(GPE_PESSOA.DTNASCPESSOA, 'DD/MM/YYYY')) / 365.25) >= :P_IDADEMINIMA OR :P_IDADEMINIMA IS NULL )
                   AND (FLOOR(TO_NUMBER(SYSDATE - TO_DATE(GPE_PESSOA.DTNASCPESSOA, 'DD/MM/YYYY')) / 365.25) <= :P_IDADEMAXIMA OR :P_IDADEMAXIMA IS NULL))
              /*AND (GPE_PESSOA.VRPRETENSAOSALARIO BETWEEN NVL(:P_SALARIOMINIMO, 0) AND NVL(:P_SALARIOMAXIMO, 999999) OR GPE_PESSOA.VRPRETENSAOSALARIO IS NULL)*/
              AND ((:P_SALARIOMINIMO IS NULL AND :P_SALARIOMAXIMO IS NULL)        
               OR EXISTS (SELECT 1
                      FROM DES_PARTICIPAPROCESSO
                     WHERE NRPARCNEGOCIOCAND = GPE_PESSOA.NRPARCNEGOCIO
                       AND (VRPRETENSAOSALARIO BETWEEN NVL(:P_SALARIOMINIMO, 0) AND NVL(:P_SALARIOMAXIMO, 999999) OR VRPRETENSAOSALARIO IS NULL)))
               $filtro
			  AND NRORG = $nrorgPadrao $filterPlaceHolder
        ORDER BY GPE_PESSOA.NMPESSOA
        ");

        if(array_key_exists('IDADEMINIMA', $parameters)){
          $idadeMinima = $parameters['IDADEMINIMA'];
          $idadeMinima = trim ($idadeMinima, "%");
        }
        else {
          $idadeMinima = '';
        }

        if(array_key_exists('IDADEMAXIMA', $parameters)){
          $idadeMaxima = $parameters['IDADEMAXIMA'];
          $idadeMaxima = trim ($idadeMaxima, "%");
        }
        else {
          $idadeMaxima = '';
        }

        if(array_key_exists('SALARIOMINIMO', $parameters)){
          $salarioMinimo = $parameters['SALARIOMINIMO'];
          $salarioMinimo = trim ($salarioMinimo, "%");
        }
        else {
          $salarioMinimo = '';
        }

        if(array_key_exists('SALARIOMAXIMO', $parameters)){
          $salarioMaximo = $parameters['SALARIOMAXIMO'];
          $salarioMaximo = trim ($salarioMaximo, "%");
        }
        else {
          $salarioMaximo = '';
        }

        $query->execute(array(
                "P_NRORG"         => $nrorg,
                "P_IDADEMINIMA"   => $idadeMinima,
                "P_IDADEMAXIMA"   => $idadeMaxima,
                "P_SALARIOMINIMO" => $salarioMinimo,
                "P_SALARIOMAXIMO" => $salarioMaximo
            )
        );

        $result = $query->fetchAll();
          return $result;
    }

    public function filtroCandidatosGeral($parameters){
      $filtroFormacao = $this->filtroFormacao($parameters);
      $filtroCandidatos = $this->filtroCandidatos($parameters);  
      $filtroExperiencia = $this->filtroExperiencia($parameters);
      $filtroCandidatosCadastrados = $this->filtroCandidatosCadastrados($parameters);

      return $filtroFormacao.$filtroCandidatos.$filtroExperiencia.$filtroCandidatosCadastrados;
    }

    public function filtroFormacao($parameters){
       $filtraFormacao = "";
        if (isset($parameters['DSINSTITUICAO']) || isset($parameters['NRAREAFORMACAO']) || isset($parameters['NRCURSO']) || isset($parameters['DSCURSO']) || isset($parameters['DES_AREAFORMACAO_NRORG']) || isset($parameters['CDIDIOM'])) {
          $dsinstituicao = '';
          $query_dsinstituicao = '';
          if(isset($parameters['DSINSTITUICAO'])){
            $dsinstituicao = $parameters['DSINSTITUICAO'];
            $query_dsinstituicao = "AND (NVL(DSINSTITUICAO, '0') LIKE '$dsinstituicao')";
          }

          $nrareaformacao = '';
          $query_nrareaformacao = '';
          if(isset($parameters['NRAREAFORMACAO'])){
            $nrareaformacao = $parameters['NRAREAFORMACAO'];
            $query_nrareaformacao = "AND (NVL(NRAREAFORMACAO, 0) = '$nrareaformacao')";
          }

          $nrcurso = '';
          $query_nrcurso = '';

          if(isset($parameters['NRCURSO'])){
            $nrcurso = $parameters['NRCURSO'];
            $query_nrcurso = "AND (NVL(NRCURSO, 0) = '$nrcurso')";
          }

          $dscurso = '';
          $query_dscurso = '';
          if(isset($parameters['DSCURSO'])){
            $dscurso = $parameters['DSCURSO'];
            $query_dscurso = "AND (NVL(DSCURSO, '0') LIKE '$dscurso')";
          }

          $des_areaformacao_nrorg = '';
          $query_des_areaformacao_nrorg = '';
          if(isset($parameters['DES_AREAFORMACAO_NRORG'])){
            $des_areaformacao_nrorg = $parameters['DES_AREAFORMACAO_NRORG'];
            $query_des_areaformacao_nrorg = "AND (NVL(DES_AREAFORMACAO_NRORG, '0') LIKE '$des_areaformacao_nrorg')";

          }

          $cdidioma = '';
          $query_cdidioma = '';
          if(isset($parameters['CDIDIOM'])){
            $cdidioma = $parameters['CDIDIOM'];
            $query_cdidioma = "AND (NVL(CDIDIOMA, 0) = '$cdidioma')";
          }

          $filtraFormacao = "
              AND EXISTS (SELECT 1
                              FROM DES_FORMACAO
                             WHERE NRORG          = GPE_PESSOA.NRORG
                               AND NRPARCNEGOCIO  = GPE_PESSOA.NRPARCNEGOCIO
                                $query_dsinstituicao 
                                $query_nrareaformacao
                                $query_nrcurso 
                                $query_dscurso 
                                $query_des_areaformacao_nrorg 
                                $query_cdidioma 
                                )
          ";
        }
        return $filtraFormacao;
    }

    public function filtroCandidatos($parameters) {
      $candidatosProcesso = '';
      $query_candidatosProcesso = '';
      if(isset($parameters['NRPARCNEGOCIOCANDIDATOS'])){
        $candidatosProcesso = $parameters['NRPARCNEGOCIOCANDIDATOS'];
        $nrparcnegocio = null;
        foreach ($candidatosProcesso as $key => $row) {
          if($key == 0){
            $nrparcnegocio = $nrparcnegocio.$row['NRPARCNEGOCIO'];
          } else{
            $nrparcnegocio = $nrparcnegocio.', '.$row['NRPARCNEGOCIO'];
          }
        }
        $query_candidatosProcesso = "AND GPE_PESSOA.NRPARCNEGOCIO IN ($nrparcnegocio)";
      } 
      return $query_candidatosProcesso;
    }

    public function filtroExperiencia($parameters){
      $filtraExperiencia = "";
      if (isset($parameters['NROCUPACAOM']) || isset($parameters['DSOCUPACAO'])) {
        $nrocupacaom = '';
        $query_nrocupacaom = '';
        if(isset($parameters['NROCUPACAOM'])){
          $nrocupacaom = $parameters['NROCUPACAOM'];
          $query_nrocupacaom = "AND (NVL(NROCUPACAOM, 0) LIKE '$nrocupacaom')";
        }

        $dsocupacao = '';
        $query_dsocupacao = '';
        if(isset($parameters['DSOCUPACAO'])){
          $dsocupacao = $parameters['DSOCUPACAO'];
          $query_dsocupacao = "AND (NVL(DSOCUPACAO, '0') LIKE '$dsocupacao')";
        }

        $filtraExperiencia = "
            AND EXISTS (SELECT 1
                          FROM GPE_EMPREGANTERIOR
                         WHERE NRORG          = GPE_PESSOA.NRORG
                           AND NRPARCNEGOCIO  = GPE_PESSOA.NRPARCNEGOCIO
                           $query_nrocupacaom 
                           $query_dsocupacao)
        ";
      }
      return $filtraExperiencia;
    }

    public function filtroCandidatosCadastrados($parameters){
      $filtrarCadastroCompleto = "";
      $value = null;
      if(isset($parameters['IDCADASTROCOMPLETO'])){
        if($parameters['IDCADASTROCOMPLETO'] == '%S%') {
          $value = 0;
        } else  if($parameters['IDCADASTROCOMPLETO'] == '%N%') {
          $value = 1;
        }
      }

      if(!is_null($value)){
        $filtrarCadastroCompleto = "
          AND (SELECT DECODE(SUM(STATUS), 5, 0, 1) STATUS
                FROM (
                SELECT
                  CASE
                    WHEN(COUNT(1) > 0)
                    THEN 1
                  END AS STATUS
                FROM DES_APRESENTACAOCAND
                WHERE NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                AND NRORG           = GPE_PESSOA.NRORG
                UNION ALL
                SELECT 
                  CASE
                    WHEN(COUNT(1) > 0)
                    THEN 1
                  END AS STATUS
                FROM ENDERECOPARC
                WHERE NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                AND NRORG           = GPE_PESSOA.NRORG
                UNION ALL
                SELECT
                  CASE
                    WHEN(COUNT(1) > 0)
                    THEN 1
                  END AS STATUS
                FROM COMUNICAPARC
                WHERE NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                AND NRORG           = GPE_PESSOA.NRORG
                AND CDFORMACOMU    != '05'
                UNION ALL
                SELECT 
                  CASE
                    WHEN (COUNT(1) > 0)
                    THEN 1
                  END AS STATUS
                FROM DES_FORMACAO
                WHERE NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                AND NRORG           = GPE_PESSOA.NRORG
                UNION ALL
                SELECT 
                  CASE
                    WHEN (COUNT(1) > 0)
                    THEN 1
                  END AS STATUS
                FROM GPE_EMPREGANTERIOR
                WHERE NRPARCNEGOCIO = GPE_PESSOA.NRPARCNEGOCIO
                AND NRORG           = GPE_PESSOA.NRORG
                )) = ".$value."
            ";
      }

      return $filtrarCadastroCompleto;
    }

    public function getCandidatos($parameters, $conditions) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
		    $nrorgPadrao = self::ORG_PADRAO;
        
        // VALIDA PARECER FINAL DO CANDIDATO
        $whereValidaParecerFinal = "";
        if (array_key_exists("P_PARECERFINAL", $parameters) && $parameters["P_PARECERFINAL"]) {
          $whereValidaParecerFinal = " AND IDPARECERFINAL IS NOT NULL";
        }
        
        // ADICIONA FILTRO POR PROCESSO
        $whereParticipaProcesso = "";
        if(array_key_exists("P_NRPROCESSO", $parameters)){
          
        $whereNotParticipaProcesso = "NOT";
        if(array_key_exists("P_IDPARTIPAPROCESSO", $parameters) && $parameters["P_IDPARTIPAPROCESSO"]){
          $whereNotParticipaProcesso = " ";
        }
        
        $nrprocesso = $parameters["P_NRPROCESSO"];
        $whereParticipaProcesso = "AND $whereNotParticipaProcesso EXISTS (SELECT 1
                               FROM DES_PARTICIPAPROCESSO
                                WHERE NRORG             = $nrorg
                                  AND NRPARCNEGOCIOCAND = PARC.NRPARCNEGOCIO
                                  AND NRPROCESSO        = $nrprocesso)";
        } else {
          $whereParticipaProcesso = "AND EXISTS (SELECT 1
                                 FROM DES_PARTICIPAPROCESSO
                                WHERE NRORG             = $nrorg
                                  AND NRPARCNEGOCIOCAND = PARC.NRPARCNEGOCIO
                                  $whereValidaParecerFinal)";
        }

        // ADICIONA FILTRO POR FASE
        $whereParticipaFase = "";
        if (array_key_exists("P_NRFASE", $parameters)) {
          $nrfase = $parameters["P_NRFASE"];
          $whereParticipaFase = "AND EXISTS (SELECT 1
                                 FROM DES_PARTICIPAFASE
                                WHERE NRORG             = $nrorg
                                  AND NRPARCNEGOCIOCAND = PARC.NRPARCNEGOCIO
                                  AND NRFASE            = $nrfase)";
        }

        $query = $this->entityManager->getConnection()->prepare("
          SELECT * FROM ( SELECT
              PARC.NRPARCNEGOCIO,
              PARC.NMPRINCIPALPARC,
              PARC.NRORG
              FROM PARCNEGOCIO PARC
              JOIN PARCERIA
                ON PARCERIA.NRPARCNEGOCIO  = PARC.NRPARCNEGOCIO
               AND PARCERIA.CDTIPOPARCEIRO = 'CANDIDATO'
               AND PARCERIA.NRORG          = PARC.NRORG
              JOIN DES_CURRICULOGERAL
                ON DES_CURRICULOGERAL.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
               AND DES_CURRICULOGERAL.NRORG         = PARC.NRORG
             WHERE PARC.IDATIVO = 'S'
               AND PARC.NRORG = $nrorgPadrao
               $whereParticipaProcesso
               $whereParticipaFase
               ORDER BY PARC.NMPRINCIPALPARC )
          WHERE 1 = 1     
          $conditions         
        ");

        $query->execute(
            array()
        );

        $result = $query->fetchAll();
        return $result;
    }

    public function adicionarCandidatoProcesso($dataset) {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();

        /** @var $participaProcesso \Folha\Entities\DesParticipaprocesso */
        $participaProcesso = $this->entityManager->getRepository('\Folha\Entities\DesParticipaprocesso')
            ->findOneBy(array('nrparcnegociocand' => $dataset["NRPARCNEGOCIO"],
                                     'nrprocesso' => $dataset["NRPROCESSO"],
                                          'nrorg' => $nrorg));

        if (!is_object($participaProcesso)) {

            $nrprocparticipa = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'DES_PARTICIPAPROCESSO', 1, 22, true);

            $this->entityManager->beginTransaction();

            $participaProcesso = new \Folha\Entities\DesParticipaprocesso();
            $participaProcesso->setNrprocparticipa($nrprocparticipa);
            $participaProcesso->setNrorg($nrorg);
            $participaProcesso->setCdstatus(self::STATUS_ANALISE);
            $participaProcesso->setNrprocesso($dataset["NRPROCESSO"]);
            $participaProcesso->setNrparcnegociocand($dataset["NRPARCNEGOCIO"]);
            $participaProcesso->setVrpretensaosalario(0);
            $participaProcesso->setDtinclusao(DateUtil::getDataAtual());

            $this->entityManager->persist($participaProcesso);
            $this->entityManager->flush();
            $this->entityManager->commit();
            $this->notificarCandidato($dataset);
        }
    }

    public function notificarCandidato($dataset){
      $nrorg = $this->hcmParameters->getNrorgPadrao();
      $nrparcnegocio = $dataset["NRPARCNEGOCIO"];
      $sql = "SELECT DISTINCT INITCAP(O.NMOPERADOR) NMOPERADOR,
              O.DSEMAILOPER
              FROM OPERADOR O
              JOIN PARCNEGOCIO P
              ON O.NRPARCNEGOCIO = P.NRPARCNEGOCIO
              WHERE (O.NRPARCNEGOCIO = '$nrparcnegocio') AND (O.NRORG = '$nrorg')";
      try {
          $std = $this->entityManager->getConnection()->prepare($sql);
          $std->execute();
          $result = $std->fetchAll();
          if (count($result[0]) > 0) {
              $nmoperador = $result[0]['NMOPERADOR'];
              $dsemailoper = $result[0]['DSEMAILOPER'];
          } else {
              if(array_key_exists('NMOPERADOR', $result[0])){
                throw new \Exception("Email do operador não encontrado.'");
              } else if(array_key_exists('DSEMAILOPER', $result[0])){
                throw new \Exception("Nome do operador não encontrado.'");
              } else {
                throw new \Exception("Nenhum dado encontrado'");
              }
          }
      }
      catch (Exception $e) {
          throw new exception ($e->getMessage());
      }
      $nrorgEstru = $this->hcmParameters->getNrorgTrabalho();
      $nrprocesso = $dataset['NRPROCESSO'];
      $sql = "SELECT NVL(NMFANTASIA, NMESTRUTURAM) NMFANTASIA, NMPROCESSO FROM DES_PROCESSO P 
              JOIN ESTRUTURAM E
              ON P.NRESTRUTGEREN = E.NRESTRUTURAM
              JOIN ESTRUTURAH H
              ON E.NRESTRUTURAM = H.NRESTRUTURAM
              WHERE (NRPROCESSO = '$nrprocesso')  AND (P.NRORG = '$nrorgEstru')";
      try {
          $std = $this->entityManager->getConnection()->prepare($sql);
          $std->execute();
          $result = $std->fetchAll();
          if (count($result[0]) > 0) {
              $nmestruturam = $result[0]['NMFANTASIA'];
              $nmprocesso = $result[0]['NMPROCESSO'];
          } else {
              if(array_key_exists('NMFANTASIA', $result[0])){
                throw new \Exception("Estrutura do processo não encontrado.'");
              } else if(array_key_exists('NMPROCESSO', $result[0])){
                throw new \Exception("Nome do processo não encontrado.'");
              } else {
                throw new \Exception("Nenhum dado encontrado'");
              }
          }
      }
      catch (Exception $e) {
          throw new exception ($e->getMessage());
      }
      $title = 'Alerta de Vagas';
      $subject = 'Alerta de Vagas';
      $content = 'Você acaba de ser adicionado ao processo seletivo da vaga <b>'.$nmprocesso.'</b> da empresa '.$nmestruturam.'.<br><br>'.'Acesse o <a href="http://talentos.pebbian.com" target="_blank">Portal de Talentos</a> para obter mais detalhes sobre esta vaga.';
      $headerContent = 'Olá'.' '.$nmoperador;
      $footerContent = null;
      $footerTemplate = 'Você está recebendo este email por ter sido relacionado em uma vaga do portal de talentos.';
      $nmfrom = 'RH Teknisa';
      $this->enviarEmailService->enviarEmailAlerta($title, $dsemailoper, $nmoperador, $nmestruturam, $subject, $content, $headerContent, $footerContent, $footerTemplate, $nmfrom);

    }

    public function removerCandidatoProcesso($dataset) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();

        $this->entityManager->beginTransaction();

        $participaProcesso = $this->entityManager->getRepository('\Folha\Entities\DesParticipaprocesso')
        ->findOneBy(array('nrparcnegociocand' => $dataset['NRPARCNEGOCIO'],
                                         'nrprocesso' => $dataset['NRPROCESSO'],
                                              'nrorg' => $nrorg));

        $this->entityManager->remove($participaProcesso);
        $this->entityManager->flush();
        $this->entityManager->commit();

    }

}