<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Calculos\Contabilizacao\CalculoContabilizacao;
use HCM\FPA\Calculos\Contabilizacao\RetrocedeContabilizacao;
use HCM\FPA\Calculos\Contabilizacao\PercorreContabilizacao;

class ContabilizacaoMensalController extends Crud {

    protected $dataSourceGetFormContabil = 'fpa_formcontabil_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERADOR', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $HCMParameters,
    percorreContabilizacao $percorreContabilizacao
){  
        $this->parameterBag = $parameterBag;
        $this->HCMParameters = $HCMParameters;
        $this->percorreContabilizacao = $percorreContabilizacao;
        parent::__construct($dataSourceManager);
    }

    public function getFormContabil (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetFormContabil);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] == 'ESTRUTURA'){
                $row['NRESTRUTLEGAL'] = $row['DSINICIO'];
                if($row['DSFIM'])
                    $row['NRESTRUTGEREN'] = $row['DSFIM'];
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceGetFormContabil, $rows));
    }

    public function setFormContabil (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_formcontabil";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMCONTABIL', $row)) {
                    if ($row['NRFORMCONTABIL'] === '') {
                       $row['NRFORMCONTABIL'] = \HCM\Util\NovoCodigo::geraCodigo($this->HCMParameters->getNrorgTrabalho(), 'FPA_FORMCONTABIL', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMCONTABIL'] = \HCM\Util\NovoCodigo::geraCodigo($this->HCMParameters->getNrorgTrabalho(), 'FPA_FORMCONTABIL', 1, 12, true); 
                }
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                $row['NRHIERARQORG'] = null;
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormContabil (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_formcontabil";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function calculoContabilizacao (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $result = $this->percorreContabilizacao->percorre($dataset[0]);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('contabilizacao', $result));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function calculoContabilizacaoLite (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);
        ini_set('memory_limit', '2048M');
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $result = $this->percorreContabilizacao->percorreLite($dataset[0]);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('contabilizacao', $result));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}