<?php

namespace HCM\FPA\ObrigacaoLegal\Dirf;

use Teknisa\Model\Gestaopessoas\Entities\FpaFormdirf;
use Teknisa\Model\Gestaopessoas\Entities\FpaDirfmagnet;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculom;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\MsgCalculo;
use HCM\FPA\ObrigacaoLegal\Dirf\QueryDirf;
use HCM\FPA\ObrigacaoLegal\Dirf\Dirfmagnet;
use HCM\FPA\Entity\FpatipomovimentEntity;


class Dirf {

    const   nomeArquivo = 'dirf.txt';   //Nome do arquivo que será gerado pela dirf
    const   ql_msg      = '<br>';       //Quebra de linha para as mensagens
    const   ql_dslinha  = "\n";         //Quebra de linha para a descrição do registro do vícnulo no magnético
    const   id13Sal     = '-13S';       //IdeNtificação de campo DIRF específico para o 13º salário (campos M13 e VALOR13)
    const   idInforme   = '-INF';       //Identificação de campo que não serão usados no TXT, mas pode-se utilizá-los no magnético/Informe de Rendimentos
    const   idFerias    = '-FERIAS';    //Identificação de campo DIRF específico para as Férias
    const   idEstagiario = '-ESTAG';    //Identificação de campo DIRF específico para Estágiario
    
    const   queryPartCdReceita = "WHEN ICF.NREVENTOM = 870 THEN '3562|'";
    const   queryPartCdReceitaNew = "WHEN LANC.NREVENTOM = 870 THEN '3562|'";
    const   VR_DIRF_RENDIMENTOS = 'VR_DIRF_RENDIMENTOS';
    const   DIRF_APURABENEFICIO = 'DIRF_APURABENEFICIO';
    const   nrTpVincProlabore = 3; //Tipo de vinculos que possuem pro-labore e sempre serao considerados no RTRT
    
    private $msgErro;           //Mensagem de erro que acumula todos os campos obrigatórios que não possuem valor
    private $queryPart;         //Fragmento de query para realizar a busca na query
    private $queryPartNew;         //Fragmento de query para realizar a busca na query
    //Formulário
    private $registrosDirf;     //Artmazenas os registros da Dirf
    private $dtCompetencia;     //Data de competência
    private $nrCenario;         //Cenário utilizado (Real, Cenário 1, Cenário 2, etc.)
    private $nrOcorren;         //Ocorrência de cálculo (geralmente é o 1). No _motor e _motor_info este parâmetro é ignorado.
    private $nrOrg;             //Nro da organização em questão
    private $nrForm;            //Número do formulário em questão. Os dados pertinentes serão extratídos daqui
    private $nmForm;            //Nome do formulário
    private $idAtivo;           //S para formulário ativo; N para não ativo
    private $idProcessaVinc;    //S para processar os vínculos; N não processa os vínculos
    private $idGeraMagnetic;    //S para gerar meio magnético (tabela)
    private $idExtinsaoEmpr;    //S indica extinsão da empresa; N empresa ativa
    private $idTipoDeclarac;    //S para Retificada; N para Original
    private $idNaturezaDecl;    //Natureza do declarante. Nros de 0 - 4, 8. Ver Manual DIRF.
    private $idOrdem;           //Ordem de classificação dos vínculos
    private $nomeInicial;       //Nome do vínculo inicial da lista
    private $nomeFinal;         //Nome do vínculo final da lista
    private $dataInicial;       //Data inicial (admissão) da lista
    private $dataFinal;         //Data final (admissão) da lista
    private $nrEstrutuLegal;    //Estrutura Legal dos vínculos
    private $nrEstrutuGeren;    //Estrutura Gerencial dos Vícnulos
    private $nrMatriInicial;    //Nro inicial da matrícula
    private $nrMatricuFinal;    //Nro final da matrícula
    private $nrPessResponsa;    //Nro da pessoa responsável
    private $nrEstrResponsa;    //Nro da estrutura da pessoa responsável
    private $nrGrupoInciden;    //Nro do grupo de incidência
    private $nrGrupoIncSaud;    //Nro do grupo de incidência para valores do plano de saúde. Em desuso. É usado no mesmo grupo de incidência, porém no tipo de incidência 99.
    private $idTpInscriResp;    //Tipo de inscrição do responsável (cpf ou cnpj)
    private $idSocioOstensi;    //S sócio ostensivo; N sócio não ostensivo
    private $idDepositJudic;    //S depósito de crédito decorrente de decisão judicial
    private $idInstitInvest;    //S instituição administradora ou intermediária de fundo opu clube de investimento
    private $idPagRendiExte;    //S pagou rendimentos a residentes ou domiciliados no exterior
    private $idPagPlanoSaud;    //S existe pagamento de valor pelo titular/dependente do plano de saúde
    private $idPagRelacCopa;    //S existe pagamento relacionado à Copa
    private $idPagRelacOlimp;   //S existe pagamento relacionado aos Jogos Olímpicos e Paraolímpicos
    private $idPagSituaEspe;    //S existe pagamento de situação especial
    private $dtEventoEspeci;    //data do evento especial (se houver)
    private $anoCalendario;
    private $anoReferencia;
    private $nrRecibo;
    private $vrMinRendimentos;
    private $beneficiariosDirf; //Array de CPFs dos beneficiarios de plano de saúde
    private $nrregimeapur;

    private $objMagnetico;
    
    private $versaoQuery;

	public function __construct($entityManager,$hcmParameters,$nrOrg, $nrForm, $dtCompetencia, $nrCenario, $nrOcorrcal, $relatoriosHcmController){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->relatoriosHcmController = $relatoriosHcmController;
        $this->msgErro = null;   //Esta variável nulo significa que não houve erros e nem falta de preenchimento
        $this->carregaDadosFormulario($nrOrg, $nrForm);
        
        if($this->nrorg == -1){ //Verifica se o carregamento de dados do formulário funcionou, caso contrário ela sai.
            return;
        }
        //$this->nrcalculofolha = FPA::determinaNrCalculoFolha(FpatipomovimentEntity::TP_MOV_MENSAL);
        //$this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculoProcesso($this->nrcalculofolha, $this->hcmParameters->getNrorgTrabalho(), 'DIRF', $this->hcmParameters->getCdOperador());
        switch($this->idOrdem){ 
            case 'ALFABETICA':
                $this->queryPart = "AND (VM.NMVINCULOM BETWEEN :inicio AND :fim)";
                $this->queryPartNew = "AND (LANC.NMVINCULOM BETWEEN :inicio AND :fim)";
                break;
            case 'DATA_ADMISSAO':
                $this->queryPart = "AND (VM.DTADMISSAOVINC BETWEEN :inicio AND :fim)";
                break;
            case 'MATRICULA':
                $this->queryPart = "AND (VM.CDMATRICULA BETWEEN :inicio AND :fim)";
                break;
            case 'ESTRUTURA':
                if ($this->dsFim) {
                    $this->queryPart = "AND (VH.NRESTRUTLEGAL = :inicio AND VH.NRESTRUTGEREN = :fim)";
                    $this->queryPartNew = "AND (LANC.NRESTRUTLEGAL = :inicio AND LANC.NRESTRUTGEREN = :fim)";
                } else {
                    $this->queryPart = "AND (VH.NRESTRUTLEGAL = :inicio AND VH.NRESTRUTGEREN  = NVL('', VH.NRESTRUTGEREN))";
                    $this->queryPartNew = "AND (LANC.NRESTRUTLEGAL = :inicio AND LANC.NRESTRUTGEREN  = NVL('', LANC.NRESTRUTGEREN))";
                }
                break;
            case 'LISTA_VINCULO':
                $inicio = is_array($this->dsInicio) ? implode(',',$this->dsInicio) : $this->dsInicio;
                $this->queryPart = "AND VM.NRVINCULOM IN ($inicio)";
                break;
            case 'LISTA_ESTRUTURA':
                $inicio = is_array($this->dsInicio) ? implode(',',$this->dsInicio) : $this->dsInicio;
                $this->queryPart = "AND VH.NRESTRUTLEGAL IN ($inicio)";
                if($fim){
                    $fim = is_array($this->dsFim) ? implode(',',$this->dsFim) : $this->dsFim;
                    $this->queryPart .= "AND VH.NRESTRUTGEREN IN ($fim)";
                }
                break; 
            default:
                $this->nrorg    = -1; //Indica falha
                //throw new \Exception( "Opção de ordenação inválida (".$this->idOrdem.").");
				$this->msgErro .= "Opção de ordenação inválida (".$this->idOrdem.").".Dirf::ql_msg;
        }
        $this->dtCompetencia   = $dtCompetencia;
        $this->nrCenario       = $nrCenario;
        $this->nrOcorren       = $nrOcorrcal;
        $this->caminhoRelativo = $caminhoRelativo.$this->nrorg.'\\';
        $this->registrosDirf   = array();
        if($this->idPagSituaEspe == 'S' && $this->dtEventoEspeci == null){
            $this->nrorg    = -1; //Indica falha
            //throw new \Exception( "A situação especial está marcada. Falta a data da situação especial.");
			$this->msgErro .= "A situação especial está marcada. Falta a data da situação especial.".Dirf::ql_msg;
        }
        if($this->idPagSituaEspe == 'N')    //Se não há situação especial não deve haver a data da mesma
            $this->dtEventoEspeci = null;
        if($this->idProcessaVinc == 'N' && $this->idGeraMagnetic == 'N'){
            $this->nrorg    = -1; //Indica falha
            //throw new \Exception( "As opções Gerar Arquivo e Gerar Magnético estão desativadas. Ative uma ou ambas opções para gerar a DIRF.");
            $this->msgErro .= "As opções Gerar Arquivo e Gerar Magnético estão desativadas. Ative uma ou ambas opções para gerar a DIRF.".Dirf::ql_msg;
        }
        $this->objMagnetico = new Dirfmagnet($nrOrg, $dtCompetencia,$this->entityManager,$this->hcmParameters);
		
		$this->vrMinRendimentos = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
		    $nrOrg, $hcmParameters->getNrorgtrabalho(), $dtCompetencia, self::VR_DIRF_RENDIMENTOS
	    );
		$this->idApurabeneficio = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
		    $nrOrg, $hcmParameters->getNrorgpadrao(), $dtCompetencia, self::DIRF_APURABENEFICIO
	    );
		
		$this->beneficiariosDirf = array();
    }
	
	//Função que carrega os dados do formulário. Na falha esta seta o nrOrg para -1 indicando a falha
    private function carregaDadosFormulario($nrOrg, $nrForm){
        $formulario = new FpaFormdirf();
        $formObj = $this->entityManager->getRepository(Repositories::FPA_FORMDIRF)->findOneBy(array("nrorg" => $nrOrg, "nrformdirf"=>$nrForm));
		//Dados do formulário
        if(is_object($formObj)) {   //Somente grava os dados se encontrou o formulário segundo a Org e seu Nro
            $this->nrorg          = $nrOrg;
            $this->nrForm         = $nrForm;
            $this->nmForm         = $formObj->getNmformdirf();
            $this->idAtivo        = $formObj->getIdativo();
            $this->idProcessaVinc = $formObj->getIdprocvinculo();
            $this->idGeraMagnetic = $formObj->getIdgerameiomag();
            $this->idExtinsaoEmpr = $formObj->getIdexcempresa();
            $this->idTipoDeclarac = $formObj->getIdtipdeclaracao();
            $this->idNaturezaDecl = $formObj->getIdnaturezadirf();
            $this->idOrdem        = $formObj->getDsordem();
            $this->dsInicio       = $formObj->getNomeinicial();
            $this->dsFim          = $formObj->getNomefinal();
			$this->nomeInicial    = $formObj->getNomeinicial(); /*pck add 24/01/19 inicio*/
            $this->nomeFinal      = $formObj->getNomefinal();
            $this->dataInicial    = $formObj->getDtinicial();
            $this->dataFinal      = $formObj->getDtfinal();
            $this->nrEstrutuLegal = $formObj->getNrestrutlegal();
            $this->nrEstrutuGeren = $formObj->getNrestrutgeren();
            $this->nrMatriInicial = $formObj->getNrmatricinicial();
            $this->nrMatricuFinal = $formObj->getNrmatricfinal(); /*pck add 24/01/19 fim*/
            $this->nrPessResponsa = $formObj->getNrpessoarespons();
            $this->nrEstrResponsa = $formObj->getNrestrutrespons();
            $this->nrGrupoInciden = $formObj->getNrgrpincide();
            $this->nrGrupoIncSaud = $formObj->getNrgrpincidsaud();
            $this->nrRecibo       = $formObj->getNrregistro();
            $this->idTpInscriResp = $formObj->getIdtpinscrespons();
            $this->idSocioOstensi = $formObj->getIdsociostens();
            $this->idDepositJudic = $formObj->getIddepojudic();
            $this->idInstitInvest = $formObj->getIdinstinvest();
            $this->idPagRendiExte = $formObj->getIdpagrendext();
            $this->idPagPlanoSaud = $formObj->getIdpagplansaud();
            $this->idPagRelacCopa = $formObj->getIdpagrelcopa();
            //$this->idPagRelacOlimp = $formObj->getIdpagrelolimp();    //DIRF 2019
            $this->idPagRelacOlimp = 'N';                               //DIRF 2019 - N – Não é fundação pública de direito privado
            //$this->idPagSituaEspe = $formObj->getIdpagsitespe();      //DIRF 2019
            $this->idPagSituaEspe = 'N';                                //DIRF 2019 - N – Não é declaração de situação especial
            $this->dtEventoEspeci = $formObj->getDteventespec();
            $this->anoCalendario  = $formObj->getNranocalen();
            $this->anoReferencia  = $formObj->getNranoref();
            $this->params = array();
            $this->params["inicio"] = $this->dsInicio;
            if($this->dsFim)
                $this->params["fim"] = $this->dsFim;
            
            // Criado valor de referencia apenas para parametrização de qual versão da DIRF usar: 1-versão utilizada até a dirf 2019, 2-versão com tabela temporária ou 3-versão com tabela de lançamentos de vínculos 
            $this->versaoQuery = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
                buscavalor($nrOrg, $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'VERSAO_QUERY_DIRF');
            
            $this->queryDirf = new QueryDirf($this->entityManager,$this->hcmParameters);
        } else {				//Formulário não encontrado
            $this->nrorg  = -1;	//indica que não carregou os dados
            $this->msgErro = 'Formulário não carregado. O número do formulário '.$nrForm.' da organização número '.$nrOrg.' não foi encontrado.'.Dirf::ql_msg;
        }
    }

	//COLEÇÃO DE REGISTROS. CADA MÉTODO GERA UM REGISTRO DO DIRF. VER MANUAL DA DIRF PARA MAIOR ESCLARECIMENTO.
	
	//DIRF 3.1 Declaração do Imposto sobre a renda retido na fonte
    private function _dirf(){
        if(is_null($this->anoReferencia) || $this->anoReferencia == '')
            //MsgCalculo::getInstance()->adicionaMensagem( "Está faltando o ano de referência.");
			$this->msgErro .= "Está faltando o ano de referência.".Dirf::ql_msg;
        if(is_null($this->anoCalendario) || $this->anoCalendario == '')
            //MsgCalculo::getInstance()->adicionaMensagem( "Está faltando o ano de calendário.");
			$this->msgErro .= "Está faltando o ano de calendário.".Dirf::ql_msg;
        if($this->idTipoDeclarac != 'S' && $this->idTipoDeclarac != 'N' )
            //MsgCalculo::getInstance()->adicionaMensagem( "Está faltando a indicação do tipo de declaração.");
			$this->msgErro .= "Está faltando a indicação do tipo de declaração.".Dirf::ql_msg;
        $anoReferencia  = str_pad(substr($this->anoReferencia,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $anoCalendario  = str_pad(substr($this->anoCalendario,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $idTipoDeclarac = str_pad(substr($this->idTipoDeclarac, 0,  1),  1,' '); //Garante o tamanho do campo em 1
        $auxNrRecibo    = $this->nrRecibo == null ? '' : str_pad(substr($this->nrRecibo, 0, 12), 12, '0', STR_PAD_LEFT);
        $registro = 'DIRF|'.$anoReferencia.'|'.$anoCalendario.'|'.$idTipoDeclarac.'|'.$auxNrRecibo.'|';
        switch ($anoReferencia) {
            case '2014':
                $idEstrutLeiaute = 'F8UCL6S|'; break;
            case '2015':
                $idEstrutLeiaute = 'M1LB5V2|'; break;
            case '2016':
                $idEstrutLeiaute = 'L35QJS2|'; break;
            case '2017':
                $idEstrutLeiaute = 'P49VS72|'; break;
			case '2018':
                $idEstrutLeiaute = 'Q84FV63|'; break;	
            case '2019':
                $idEstrutLeiaute = 'T17BS45|'; break;
            case '2020':
                $idEstrutLeiaute = 'AT65HD8|'; break;
            case '2021':
                $idEstrutLeiaute = 'VR4QLM8|'; break;
            default:
                $idEstrutLeiaute = '|'; break;
        }
        return $registro.$idEstrutLeiaute;
    }

    //RESPO 3.2 Responsável pelo preenchimento
	private function _respo(){
        $params = array();
        $params['nrorg']            = $this->nrorg;
        $params['nrpessresponsa']   = $this->nrPessResponsa;
        $params['dtcompetencia']    = $this->dtCompetencia;
        try{
            $sqlResult = $this->queryDirf->get_respo($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo RESPO, item 3.2.');
        }
        $registro = '';     //Armazena o registro
        foreach($sqlResult as $val){
            if($val['NRCPF']      == '|')   //'IFs' para verificação de campos não preenchidos
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem CPF.");
				$this->msgErro .= "Responsável pela DIRF sem CPF.".Dirf::ql_msg;
            if($val['NMNOMERESP'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem nome.");
				$this->msgErro .= "Responsável pela DIRF sem nome.".Dirf::ql_msg;
            if($val['NRDDD']      == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem Nº de DDD.");
				$this->msgErro .= "Responsável pela DIRF sem Nº de DDD.".Dirf::ql_msg;
            if($val['NRTELEFONE'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem Nº de telefone.");
				$this->msgErro .= "Responsável pela DIRF sem Nº de telefone.".Dirf::ql_msg;
            $registro = $val['NMIDENTIFICADOR'].$val['NRCPF'].$val['NMNOMERESP'].$val['NRDDD'].$val['NRTELEFONE'].$val['NRRAMAL'].$val['NRFAX'].$val['NMEMAIL'];
            if($this->idGeraMagnetic == 'S')
                $this->objMagnetico->addRegCabecal($val['CDINTESTRUTURA'], null, $val['NRPESSOA'], $this->idAtivo);
        }
        return $registro;
    }

    //DECPJ 3.4 Registro de identificação do declarante pessoa juridica
	private function _decpj(){
		/*Atualizações 2018 (em relação a 2017): itens 12 - 14*/
        $params = array(
        "idnaturezadecl"   => $this->idNaturezaDecl,
        "idsocioostensi"   => $this->idSocioOstensi,
        "iddepositjudic"   => $this->idDepositJudic,
        "idinstitinvest"   => $this->idInstitInvest,
        "idpagrendiexte"   => $this->idPagRendiExte,
        "idpagplanosaud"   => $this->idPagPlanoSaud,
        "idpagrelaccopa"   => $this->idPagRelacCopa,
        "idpagrelacolimp"  => $this->idPagRelacOlimp,
        "idpagsituaespe"   => $this->idPagSituaEspe,
        "dteventoespeci"   => $this->dtEventoEspeci,
        "nrorg"            => $this->nrorg,
        "nrestrresponsa"   => $this->nrEstrResponsa,
        "dtcompetencia"    => $this->dtCompetencia
        );
        try{
            $sqlResult = $this->queryDirf->get_decpj($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo DECPJ, item 3.4.');
        }
        $registro = '';
        foreach($sqlResult as $val){
            if($val['CDCNPJ']          == '|')   //'IFs' para verificação de campos não preenchidos
                //MsgCalculo::getInstance()->adicionaMensagem( "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem CNPJ.");
				$this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem CNPJ.".Dirf::ql_msg;
            if($val['NOMEEMPRESARIAL'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem razão social.");
				$this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem razão social.".Dirf::ql_msg;
            if($val['IDNATUREZADECLA'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Não há natureza do declarante (formulário, estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Não há natureza do declarante (formulário, estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['CDCPFRESP']       == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável perante o CNPJ não cadastrado ou sem CPF (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Responsável perante o CNPJ não cadastrado ou sem CPF (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDSOCIOSTENS']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de sócio ostensivo (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de sócio ostensivo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDDEPOJUDIC']     == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de depósito de crédito decorrente de decisão judicial (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de depósito de crédito decorrente de decisão judicial (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDINSTINVEST']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de declarante de instituição administradora (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de declarante de instituição administradora (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGRENDEXT']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de rendimentos pagos a residentes no exterior (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de rendimentos pagos a residentes no exterior (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGPLANSAUD']   == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de plano privado de assistência à saúde (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de plano privado de assistência à saúde (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if ($this->anoReferencia == 2016 || $this->anoReferencia == 2018 || $this->anoReferencia == 2019 || $this->anoReferencia == 2020 || $this->anoReferencia == 2021) {
                if($val['IDPAGRELCOPA']    == '|')
                    //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de pagamentos relacionados à Copa das Confedêrações e do Mundo (estrutura nº ".$this->nrEstrResponsa.").");
					$this->msgErro .= "Falta indicador de pagamentos relacionados à Copa das Confedêrações e do Mundo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            } else {
                unset($val['IDPAGRELCOPA']);
            }        
            if ($this->anoReferencia == 2016 || $this->anoReferencia == 2017 || $this->anoReferencia == 2018 || $this->anoReferencia == 2019 || $this->anoReferencia == 2020 || $this->anoReferencia == 2021) {
                if ($val['IDPAGRELOLIMP'] == '|')
                    //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de pagamentos relacionados aos Jogos Olímpicos e Paraolímpicos (estrutura nº " . $this->nrEstrResponsa . ")." . Dirf::ql_msg);
					$this->msgErro .= "Falta indicador de pagamentos relacionados aos Jogos Olímpicos e Paraolímpicos (estrutura nº " . $this->nrEstrResponsa . ")." . Dirf::ql_msg;
            } else {
                unset($val['IDPAGRELOLIMP']);
            }
            if($val['IDPAGSITESPE']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta indicador de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGSITESPE'] == '|S' && $val['DTEVENTDECPJ'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta data de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").");
				$this->msgErro .= "Falta data de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            foreach ($val as $field=>$res) {
                if($field != 'NRREGIMEAPUR')
                $registro .= $res;
            }
            /* Tipo de Regime de Apuração (Competência ou Caixa) */
            $this->nrregimeapur = $val['NRREGIMEAPUR'];
        }
        return $registro;
    }

    //IDREC 3.5, BPFDEC 3.6, 4.6 campos de registros
    private function _motor(){
        $idDecTerceiro      = self::id13Sal;
        $queryPartCdReceita = self::queryPartCdReceita;
        $queryPartCdReceitaNew = self::queryPartCdReceitaNew;
        $idInforme          = self::idInforme;
        $idFerias           = self::idFerias;
        $idEstagiario       = self::idEstagiario;
		$qryPartDtocorrencia = ''; //Parte para determinar se é regima caixa (shift 1 mês) ou competência
        $params = array(
            "inicio"            => $this->dsInicio,
            "fim"               => $this->dsFim,
            "nrorg"             => $this->nrorg,
            "dtcompetencia"     => $this->dtCompetencia,
            "nrgrupoinciden"    => $this->nrGrupoInciden,
            "nrcenario"         => $this->nrCenario,
            "nrocorren"         => $this->nrOcorren
        );
        
        switch($this->nrorg){
            case 1019:
                $qryPartDtocorrencia = "AND CF.DTOCORRENCIA BETWEEN DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                              1, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                              4, TO_DATE('$this->dtCompetencia'),
                                                                                              TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'))
                                                     --AND DECODE(CF.NRTIPOMOVIMENT, 1, ADD_MONTHS('$this->dtCompetencia', -1), '$this->dtCompetencia')
                                                     AND DECODE(CF.NRTIPOMOVIMENT, 1, DECODE(TO_CHAR(VM.DTRESCISAOVINC, 'MMYYYY'), TO_CHAR(CF.DTOCORRENCIA,'MMYYYY'), '$this->dtCompetencia', ADD_MONTHS('$this->dtCompetencia', -1)), '$this->dtCompetencia')";
                //$qryPartMes          = "WHEN CF.NRTIPOMOVIMENT = 1 AND (VM.DTRESCISAOVINC IS NULL OR TO_CHAR(CF.DTOCORRENCIA,'MMYYYY') <> TO_CHAR(VM.DTRESCISAOVINC,'MMYYYY')) THEN SUBSTR('$this->dtCompetencia', 7, 4) || TO_CHAR(ADD_MONTHS(CF.DTOCORRENCIA, 1),'MM')";
                break;
            default:
                $qryPartDtocorrencia = "AND CF.DTOCORRENCIA BETWEEN DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'),-1),
                                                      4, TO_DATE('$this->dtCompetencia'),
                                                      TRUNC(TO_DATE('$this->dtCompetencia' ),'YEAR'))
				            AND '$this->dtCompetencia'";
        }
        /*Ajusta a querya para regime competência ou caixa*/
		$regimeQryPart = array();
        if($this->nrregimeapur == '2') {    //Regime Caixa
            $regimeQryPart[1]	= "/*2 1*/ WHEN ICF.NREVENTOM = 1390 THEN TO_CHAR(CF.DTOCORRENCIA, 'yyyymm')
                                   WHEN CF.NRTIPOMOVIMENT = 1 AND (VM.DTRESCISAOVINC IS NULL OR TO_CHAR(CF.DTOCORRENCIA,'MMYYYY') <> TO_CHAR(VM.DTRESCISAOVINC,'MMYYYY')) THEN SUBSTR(TO_DATE('$this->dtCompetencia'/*:P_COMPETENCIA*/), 7, 4) || TO_CHAR(ADD_MONTHS(CF.DTOCORRENCIA, 1),'MM')";
            $regimeQryPart[2]   = "/*2 2*/ WHEN GIT.NMITEM = 'RTRT-ADIANT-CX' THEN TO_CHAR(ICF.DTOCORRENCIA, 'yyyymm')
                                   WHEN GIT.NMITEM = 'RTRT-ADIANT-CX-ANT' THEN TO_CHAR(ADD_MONTHS(ICF.DTOCORRENCIA, 1), 'yyyymm')";
            $regimeQryPart[3]   = "/*2 3*/ WHEN GIT.NMITEM IN ('RTRT-ADIANT-CX') /*Subtrai o adiantamento do mês para entrar no mês anterior*/
                                   THEN DECODE(IEV.IDINCIDEVENTO, 'N', 0, DECODE(IEV.IDINCIDEVENTO, '-', -1 * ICF.VREVENTOFOLHA, ICF.VREVENTOFOLHA)) * (-1)";
            $regimeQryPart[4]   = "/*2 4*/ UNION ALL
                                  SELECT GIT.NRORG, GIT.NRITEM, 'RTRT-ADIANT-CX-ANT' AS NMITEM, GIT.NRGRPINCIDE, GIT.NRTIPOINCIDE
                                    FROM FPA_GRPITEM GIT
                                   WHERE (GIT.NRORG = $this->nrorg /*:P_NRORG*/ OR GIT.NRORG = 0)
                                     AND GIT.NRGRPINCIDE  = $this->nrGrupoInciden /*:P_NRGRPDIRF*/
                                     AND GIT.NMITEM = 'RTRT-ADIANT-CX'";
            $regimeQryPart[5]   = "/*2 5*/ OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX', 10)
                                   OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX-ANT', 10)";
            $regimeQryPart[6]   = "/*2 6*/ AND CF.DTOCORRENCIA BETWEEN DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     10, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     1, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     4, TO_DATE('$this->dtCompetencia'),
                                                                                     TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'))
                                                     --AND DECODE(CF.NRTIPOMOVIMENT, 1, ADD_MONTHS('$this->dtCompetencia', -1), '$this->dtCompetencia')
                                                     AND DECODE(CF.NRTIPOMOVIMENT, 1, DECODE(TO_CHAR(VM.DTRESCISAOVINC, 'MMYYYY'), TO_CHAR(CF.DTOCORRENCIA,'MMYYYY'), '$this->dtCompetencia', ADD_MONTHS('$this->dtCompetencia', -1)), '$this->dtCompetencia')";
            /*New*/
            $regimeQryPartNew[1]	= "/*2 1*/ WHEN LANC.NREVENTOM = 1390 THEN TO_CHAR(LANC.DTOCORRENCIACALC, 'yyyymm')
                                   WHEN LANC.NRTIPOMOVIMENT = 1 AND (LANC.DTRESCISAOVINC IS NULL OR TO_CHAR(LANC.DTOCORRENCIACALC,'MMYYYY') <> TO_CHAR(LANC.DTRESCISAOVINC,'MMYYYY')) THEN SUBSTR(TO_DATE('$this->dtCompetencia'/*:P_COMPETENCIA*/), 7, 4) || TO_CHAR(ADD_MONTHS(LANC.DTOCORRENCIACALC, 1),'MM')";
            $regimeQryPartNew[2]   = "/*2 2*/ WHEN GIT.NMITEM = 'RTRT-ADIANT-CX' THEN TO_CHAR(LANC.DTOCORRENCIA, 'yyyymm')
                                   WHEN GIT.NMITEM = 'RTRT-ADIANT-CX-ANT' THEN TO_CHAR(ADD_MONTHS(LANC.DTOCORRENCIA, 1), 'yyyymm')";
            $regimeQryPartNew[3]   = "/*2 3*/ WHEN GIT.NMITEM IN ('RTRT-ADIANT-CX') /*Subtrai o adiantamento do mês para entrar no mês anterior*/
                                   THEN DECODE(IEV.IDINCIDEVENTO, 'N', 0, DECODE(IEV.IDINCIDEVENTO, '-', -1 * LANC.VREVENTOFOLHA, LANC.VREVENTOFOLHA)) * (-1)";
            $regimeQryPartNew[4]   = "/*2 4*/ UNION ALL
                                  SELECT GIT.NRORG, GIT.NRITEM, 'RTRT-ADIANT-CX-ANT' AS NMITEM, GIT.NRGRPINCIDE, GIT.NRTIPOINCIDE
                                    FROM FPA_GRPITEM GIT
                                   WHERE (GIT.NRORG = $this->nrorg /*:P_NRORG*/ OR GIT.NRORG = 0)
                                     AND GIT.NRGRPINCIDE  = $this->nrGrupoInciden /*:P_NRGRPDIRF*/
                                     AND GIT.NMITEM = 'RTRT-ADIANT-CX'";
            $regimeQryPartNew[5]   = "/*2 5*/ OR LANC.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX', 10)
                                   OR LANC.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT-ADIANT-CX-ANT', 10)";
            $regimeQryPartNew[6]   = "/*2 6*/ AND LANC.DTOCORRENCIACALC BETWEEN DECODE(LANC.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     10, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     1, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'), -1),
                                                                                     4, TO_DATE('$this->dtCompetencia'),
                                                                                     TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'))
                                                     --AND DECODE(LANC.NRTIPOMOVIMENT, 1, ADD_MONTHS('$this->dtCompetencia', -1), '$this->dtCompetencia')
                                                     AND DECODE(LANC.NRTIPOMOVIMENT, 1, DECODE(TO_CHAR(LANC.DTRESCISAOVINC, 'MMYYYY'), TO_CHAR(LANC.DTOCORRENCIACALC,'MMYYYY'), '$this->dtCompetencia', ADD_MONTHS('$this->dtCompetencia', -1)), '$this->dtCompetencia')";
        } else {                            //Regime Competência
            $regimeQryPart[1] = "";
            $regimeQryPart[2] = "";
            $regimeQryPart[3] = "";
            $regimeQryPart[4] = "";
            $regimeQryPart[5] = "";
            $regimeQryPart[6] = "/*1 6*/ AND CF.DTOCORRENCIA BETWEEN DECODE(CF.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'),-1),
                                                      4, TO_DATE('$this->dtCompetencia'),
                                                      TRUNC(TO_DATE('$this->dtCompetencia' ),'YEAR'))
                              AND '$this->dtCompetencia'";
            /*New*/
            $regimeQryPartNew[1] = "";
            $regimeQryPartNew[2] = "";
            $regimeQryPartNew[3] = "";
            $regimeQryPartNew[4] = "";
            $regimeQryPartNew[5] = "";
            $regimeQryPartNew[6] = "/*1 6*/ AND LANC.DTOCORRENCIACALC BETWEEN DECODE(LANC.NRTIPOMOVIMENT, 2, ADD_MONTHS(TRUNC(TO_DATE('$this->dtCompetencia'),'YEAR'),-1),
                                                      4, TO_DATE('$this->dtCompetencia'),
                                                      TRUNC(TO_DATE('$this->dtCompetencia' ),'YEAR'))
                              AND '$this->dtCompetencia'";
        }
        try{
            if($this->versaoQuery == 1) {
                $sqlResult = $this->queryDirf->get_motor($params,$this->queryPart,$idDecTerceiro,$queryPartCdReceita,$idInforme,$idFerias,$idEstagiario,$regimeQryPart);
            } else if ($this->versaoQuery == 3) {
                $sqlResult = $this->queryDirf->get_motor_hcm($params,$this->queryPart,$this->queryPartNew,$idDecTerceiro,$queryPartCdReceitaNew,$idInforme,$idFerias,$idEstagiario,$regimeQryPartNew);
            } else {
                $sqlResult = $this->queryDirf->get_motor_new($params,$this->queryPart,$this->queryPartNew,$idDecTerceiro,$queryPartCdReceitaNew,$idInforme,$idFerias,$idEstagiario,$regimeQryPartNew);
            }
        } catch(\Exception $e){
           throw new \Exception('Erro na query, campos IDREC, BPFDEC e campos de registros, itens 3.5, 3.6 e 4.6. ('.$e->getMessage().')');
        }
        
        if(count($sqlResult) == 0){ 
            throw new \Exception( "Não há registros para os parâmetros selecionados.");
        }
        $idrecAntes      = '';      //Armazena os valores anteriores
        $bpfdecAntes     = '';      //Armazena os valores anteriores
        $registrosDirf   = array(); //Registros para o arquivo da DIRF
        $dsLinhaRegistro = '';      //Para alimentar o magnético

        foreach($sqlResult as $val){    //Laço para capturar todos os registros e formar
            if($val['FLAG_TXT'] == 'S' && ($val['TOTAL'] + $val['TOTAL13'] > 0)
                && ( (!$this->vrMinRendimentos || (float)$val['TOTAL_RTRT']  >= (float)$this->vrMinRendimentos )
                  || (!$this->vrMinRendimentos || (float)$val['TOTAL_RTIRF'] > 0)
                  || (int)$val['NRTIPOVINCULOM'] == self::nrTpVincProlabore) ) {

                //todo verificar se o CPF em questão possui dependentes para pensão alimentícia e criar/ajustar o campo 5 (Indicador de identificação do alimentando)
                $this->beneficiariosDirf[$val['NRCPFPESSOA']] = 1; //Armazena os CPFs dos participantes da DIRF
                $idrec = $val['IDREC'] . $val['CDRECEITA'];
                $bpfdec = $val['BPFDEC'] . $val['CDCPF'] . $val['NOME'] . $val['DTMOLESTIA'] . '{idinfpa}' . $val['IDPREVCOMP'];
                if($val['NMITEM'] == 'RIO') {
                    $reg = $val['CDREGISTRO'] . $val['ANO'] . 'RENDIMENTOS ISENTOS - OUTROS|';
                }else{
                    $reg = $val['CDREGISTRO'] . $val['M01'] . $val['M02'] . $val['M03'] . $val['M04'] . $val['M05'] . $val['M06'] . $val['M07'] . $val['M08'] . $val['M09'] . $val['M10'] . $val['M11'] . $val['M12'] . $val['M13'];
                }
                if ($idrec != $idrecAntes) {          //Verifica se houve novo registro do tipo IDREC
                    $idrecAntes = $idrec;          //Atualiza o IDREC corrente
                    $bpfdecAntes = '';              //Para garantir que o mesmo gere uma linha BPFDEC
                    array_push($registrosDirf, $idrec); //Adiciona o registro IDREC (somente uma vez)
                }
                if ($bpfdec != $bpfdecAntes) {            //Verifica se houve um novo registro do tipo BPFDEC
                    if ($val['CDCPF'] == '|')    //'IFs' para verificação de campos não preenchidos
						$this->msgErro .= "Vínculo " . $val['NMPESSOA'] . " sem CPF." . Dirf::ql_msg;   //$this->msgErro .= "Vínculo (".$val['NRVINCULOM'].", ".$val['NMVINCULOM'].") sem CPF.".Dirf::ql_msg;
                    if ($val['NOME'] == '|')
						$this->msgErro .= "Vínculo de CPF " . $val['NRCPFPESSOA'] . " sem nome." . Dirf::ql_msg;  //$this->msgErro .= "Vínculo ".$val['NRVINCULOM']." sem nome.".Dirf::ql_msg;
                    if ($val['CDRECEITA'] == '|')
						$this->msgErro .= "Vínculo " . $val['NMPESSOA'] . " de CPF " . $val['NRCPFPESSOA'] . " sem código de receita DIRF ou sem vínculo empregatício." . Dirf::ql_msg;
                    $bpfdecAntes = $bpfdec;             //Atualiza o BPFDEC corrente
                    $infpa = $this->_infpa($val['NRCPFPESSOA']); //Procura por registros INFPA
                    if(count($infpa) > 0){
                        array_push($registrosDirf, str_replace('{idinfpa}', 'S|', $bpfdec));    //Adiciona o registro BPFDEC (somente uma vez)
                        foreach($infpa as $aux){
                            array_push($registrosDirf, $aux);
                        }
                    } else {
                        array_push($registrosDirf, str_replace('{idinfpa}', 'N|', $bpfdec));    //Adiciona o registro BPFDEC (somente uma vez)
                    }
                }
                array_push($registrosDirf, $reg);       //Adiciona o registro
                if($this->idGeraMagnetic == 'S' && $this->msgErro == null) { //Somente gera o final do magnético se o mesmo foi solicitado, se existem registros e não houver erros
                    $dsLinhaRegistro = array(
                        'IDREC' => $idrecAntes,
                        'BPFDEC' => $bpfdecAntes,
                        'REG' => $reg
                    );
                    $this->objMagnetico->addRegVinculo2($val['NRCPFPESSOA'], $val['NMPESSOA'], null, $val['NMITEM'], $val['CDRECEITA'], $val['TOTAL'], $val['TOTAL13'], $dsLinhaRegistro, $val['COMPLEMENTO'], $val['NRESTRUTLEGAL']);
                }
            }
        }
        return $registrosDirf;
    }

	//INFPA 3.17 Registro de informações do beneficiário da pensão alimentícia
    private function _infpa($cpfTitular){
        $params = array(
        "cpftitular"    => $cpfTitular,
        "nrorg"         => $this->nrorg,
        "dtcompetencia" => $this->dtCompetencia
        );
        try{
			$sqlResult = $this->queryDirf->get_infpa_aux($params);
		} catch(\Exception $e){
			throw new \Exception('Erro na query auxiliar, campo INFPA, iten 3.17.');
		}
        $registros = array();
		if(count($sqlResult[0]) > 0) { //Testa se existe dependentes
			try{
				$sqlResult = $this->queryDirf->get_infpa($params);
			} catch(\Exception $e){
				throw new \Exception('Erro na query, campo INFPA, iten 3.17.');
			}
			foreach($sqlResult as $reg){
				if ($reg) {
					array_push($registros, $reg['INFPA'].$reg['CDCPFDEP'].$reg['DTNASCDEP'].$reg['NOMEDEP'].$reg['RELACDEP']);
				}
			}
		}
		
        return $registros;
    }

    //PSE 5 Registro de pagamentos a plano privado de assitência à saúde - coletivo empresarial
	private function _pse(){
        $params = array(
        "nrorg"             => $this->nrorg,
        "inicio"            => $this->dsInicio,
        "fim"               => $this->dsFim,
        "dtcompetencia"     => $this->dtCompetencia,
        "nrgrupoinciden"    => $this->nrGrupoInciden,
        "nrcenario"         => $this->nrCenario,
        "nrocorren"         => $this->nrOcorren
        );
        try{
            /*Ajusta a query para regime competência ou caixa*/
            if($this->nrregimeapur == '2') {    //Regime Caixa
                $queryPartPse = "AND CF.DTOCORRENCIA BETWEEN ADD_MONTHS(TRUNC(TO_DATE( '$this->dtCompetencia' ), 'YEAR'), -1) AND ADD_MONTHS( '$this->dtCompetencia', -1)";
                $queryPartPseNew = "AND LANC.DTOCORRENCIACALC BETWEEN ADD_MONTHS(TRUNC(TO_DATE( '$this->dtCompetencia' ), 'YEAR'), -1) AND ADD_MONTHS( '$this->dtCompetencia', -1)";
            } else {
                $queryPartPse = "AND CF.DTOCORRENCIA BETWEEN TRUNC( TO_DATE( '$this->dtCompetencia' ), 'YEAR' ) AND '$this->dtCompetencia'";
                $queryPartPseNew = "AND LANC.DTOCORRENCIACALC BETWEEN TRUNC( TO_DATE( '$this->dtCompetencia' ), 'YEAR' ) AND '$this->dtCompetencia'";
            }
            if($this->idApurabeneficio == 1){
                if($this->versaoQuery == 1) {
                    $sqlResult = $this->queryDirf->get_pse_apurabeneficio($params,$this->queryPart,$queryPartPse);
                } else if ($this->versaoQuery == 3) {
                    $sqlResult = $this->queryDirf->get_pse_apurabeneficio_hcm($params,$this->queryPart,$queryPartPse); //Criação para melhora de performance olhando para tabela temporaria
                } else {
                    $sqlResult = $this->queryDirf->get_pse_apurabeneficio2($params,$this->queryPart,$queryPartPse); //Criação para melhora de performance olhando para tabela temporaria
                }
            } else {
                if($this->versaoQuery == 1) {
                    $sqlResult = $this->queryDirf->get_pse($params,$this->queryPart,$queryPartPse);
                } else if ($this->versaoQuery == 3) {
                    $sqlResult = $this->queryDirf->get_pse_hcm($params,$this->queryPartNew,$queryPartPseNew);
                } else {
                    $sqlResult = $this->queryDirf->get_pse_new($params,$this->queryPartNew,$queryPartPseNew);
                }
            }
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo PSE, item 5. ('.$e->getMessage().')');
        }
        $pse = '';
        $opseAntes  = '';    //Armazena os valores anteriores
        $tpseAntes = '';      
        $cpfTitular = '';
        $registros = array();
        
        foreach($sqlResult as $val){

            if(isset($this->beneficiariosDirf[$val['NRCPFTITULAR']])) {

                if ($pse == '') {
                    $pse = $val['PSE'];
                    array_push($registros, $pse);    //Adiciona o registro PSE (somente uma vez)
                }
                
                $opse = $val['OPSE'] . $val['CNPJ_EMPR'] . $val['NOME_EMPR'] . $val['REG_ANS'];
                $tpse = $val['TPSE'] . $val['CDCPF'] . $val['NOME'] . $val['VALOR'];
                $tpseref = $val['TPSE'] . $val['CDCPF'] . $val['NOME'];
                $dtpse = $val['DTPSE'] . $val['CDCPFDEP'] . $val['DTNASCDEP'] . $val['NOMEDEP'] . $val['RELACDEP'] . $val['VALORDEP'];
                $isDep   = $val['NOMEDEP'] != '|';      //Flag para indicação que o TPSE / registro atual se trata de um dependente

                if ($opse != $opseAntes) {              //Verifica se houve novo registro do tipo OPSE
                    if ($val['CDCNPJESTRUT'] == '|')    //'IFs' para verificação de campos não preenchidos
                        $this->msgErro .= "Estrutura (" . $val['NMRAZSOCESTRUT'] . ") sem CNPJ." . Dirf::ql_msg;
                    $opseAntes = $opse;                 //Atualiza o OPSE corrente
                    $tpseAntes = '';                    //Força a utilização do TPSE já que o OPSE mudou
                    array_push($registros, $opse);      //Adiciona o registro OPSE
                }

                if ($tpse != $tpseAntes && !$isDep) {       //Verifica se houve novo registro do tipo TPSE para TITULAR
                    $tpseAntes  = $tpse;                    //Atualiza o TPSE corrente
                    $cpfTitular = $val['NRCPFTITULAR'];     //Registra CPF do titular para identificação na parte de dependentes DTPSE
                    array_push($registros, $tpse);          //Adiciona o registro TPSE
                }

                if ($isDep && $cpfTitular == $val['NRCPFTITULAR']) {    //É dependente do titular corrente?
                    array_push($registros, $dtpse); 
                }

            }

        }
        if (!empty($registros)) { //Caso tenha gerado plano(s) de saúde altera o identificador do registro DECPJ
            $array = explode('|',$this->registrosDirf[2]); // Quebra os campos do registro DECPJ em um array.
            $array[9] = 'S';                               // Altera o identificador de plano de saúde (posição 10) para 'S'.
            $this->registrosDirf[2] = implode('|',$array); // Retorna com o registro DECPJ com o identificador de plano de saúde alterado.
        }
        return $registros;
    }
	
    //INF 5.7 Registro de informações complementares para o comprovante de rendimento (baiscamente o plano de saúde)
	private function _motor_inf(){
        $params = array(
        "nrorg"             => $this->nrorg,
        "inicio"            => $this->dsInicio,
        "fim"               => $this->dsFim,
        "dtcompetencia"     => $this->dtCompetencia,
        "nrgrupoinciden"    => $this->nrGrupoInciden,
        "nrcenario"         => $this->nrCenario,
        "nrocorren"         => $this->nrOcorren,
        "querypart"         => $this->queryPart
        );
        try{
            if($this->versaoQuery == 1) {
                $sqlResult = $this->queryDirf->get_motor_inf($params,$this->queryPart);
            } else if ($this->versaoQuery == 3) {
                $sqlResult = $this->queryDirf->get_motor_inf_hcm($params,$this->queryPartNew);
            } else {
                $sqlResult = $this->queryDirf->get_motor_inf_new($params,$this->queryPartNew);
            }
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo INF, iten 5.7.');
        }
        $infAntes = '';
        $informacoes = '';
        $registros = array();
        foreach($sqlResult as $val){
			if(isset($this->beneficiariosDirf[$val['NRCPFPESSOA']])) {
                if ($val['CDCPF'] == '|')   //'IFs' para verificação de campos não preenchidos
                    $this->msgErro .= "Vínculo (" . $val['NMVINCULOM'] . ") sem CPF." . Dirf::ql_msg;
	            
	            $inf = $val['INF'] . $val['CDCPF'];
	            if($infAntes === ''){
	                $infAntes = $inf;
	            }
	            if ($inf !== $infAntes) {
	                array_push($registros, $infAntes . $informacoes . '|');
	                $infAntes = $inf;
	                $informacoes = '';
	            }
	            $informacoes .= ($informacoes == '' ? '' : ', ') . $val['COMPLEMENTO'];
	        }
		}
        if ($inf) {
            array_push($registros, $inf . $informacoes . '|');
        }
        return $registros;
    }

    private function _fimdirf(){
        return 'FIMDIRF|';
    }
    
    private function geraRegistrosDirf(){   //Gera em um vetor os registros da DIRF para o arquivo e o magnético
        $this->registrosDirf = array();
        array_push($this->registrosDirf, $this->_dirf());    //DIRF 3.1 Declaração do Imposto sobre a renda retido na fonte
        array_push($this->registrosDirf, $this->_respo());   //RESPO 3.2 Responsável pelo preenchimento
        array_push($this->registrosDirf, $this->_decpj());   //DECPJ 3.4 Registro de identificação do declarante pessoa juridica
        foreach($this->_motor() as $reg){                    //IDREC 3.5, BPFDEC 3.6, 4.6 campos de registros
            array_push($this->registrosDirf, $reg);
        }
        if($this->idGeraMagnetic == 'S' && $this->msgErro == null) { //Somente gera o final do magnético se o mesmo foi solicitado, se existem registros e não houver erros
            $this->objMagnetico->fimAddRegVinculo();
        }
        foreach($this->_pse() as $reg){                       //PSE 5 Registro de pagamentos a plano privado de assistência à saúde - coletivo empresarial
            array_push($this->registrosDirf, $reg);
        }
        foreach($this->_motor_inf() as $reg){                 //INF 5.7 Registro de informações complementares para o comprovante de rendimento
            array_push($this->registrosDirf, $reg);
        }
        array_push($this->registrosDirf, $this->_fimdirf()); //FIMDirf 5.8 Registro indicador do término da declaração
        /*MsgCalculo::getInstance()->salvaMensagens(true);
        $this->mensagens = MsgCalculo::getInstance()->retornaMensagens();
        MsgCalculo::getInstance()->clear();*/
    }

    public function gerar() {
        if($this->nrorg == -1) 
            return false;
        
        if($this->idGeraMagnetic == 'S')
            $this->geraRegistrosDirf(); 

        if(!is_null($this->msgErro))    
            return false;               

        if($this->idProcessaVinc == 'S'){  
            foreach($this->registrosDirf as $reg){                            
                FPA::escreveArquivo(Dirf::nomeArquivo,$reg.PHP_EOL);
            }  
            return FPA::getTempDirectory().'/'.Dirf::nomeArquivo;
        }
        /*
        if($this->idGeraMagnetic == 'S'){   
            $param = array();
            $param['P_COMPETENCIA'] = \FPA::getDtCompetencia();
            $param['P_NRORG'] = \FPA::getNrorgTrabalho();
            $param['P_ANOREF'] = tk_get_field_value('NRANOREF');
            $param['P_ANOCALEN'] = tk_get_field_value('NRANOCALEN');
            $param['P_CONFIDENCIAL'] = \Folha\Facade\Relatorios\Confidencial::setQueryConfidencial();
            $param['CDOPERADOR'] = \Geral\Util\Ambiente::getCdOperador();
            $param['NMOPERADOR'] = \Geral\Util\Ambiente::getNmOperador();
            tk_open_report_from_array('informe_rendimentos', $param, \Setforms2\Runtime\Helper\Report\TkBirtReport::FORMAT_PDF);
        }*/
        return false;
    }

    public function retroceder(){
        if($this->nrorg == -1) {    
            MsgCalculo::getInstance()->adicionaMensagem( "Não foi possível retroceder a Dirf pois o formulário não está carregado.");
            return false;           
        }
        $cont = $this->objMagnetico->excluirCompetencia();
        if($cont == 0) {        
            MsgCalculo::getInstance()->adicionaMensagem( "Não havia nenhum registro no magnético a retroceder da Organização ".$this->nrorg." na competência ".$this->dtCompetencia);
            return false;       
        }
        return $cont;           
    }

    public function getQdeRegsMagnet(){   
        if($this->nrorg == -1) 
            return false;
        if($this->idGeraMagnetic == 'S')
            return $this->objMagnetico->qdeRegistros();
        return 0;
    }
    
    public function getMensagens(){
        return is_null($this->msgErro)?false:$this->msgErro;
    }
    
    public function geraRelatorio(){
        $params = array(
        "nrorg"             => $this->nrorg,
        "inicio"            => $this->dsInicio,
        "fim"               => $this->dsFim,
        "dtcompetencia"     => $this->dtCompetencia,
        "querypart"         => $this->queryPart
        );
        
        $vinculos = $this->queryDirf->get_nrvinculos($params,$this->queryPart);
        $vinculos_str;
        foreach($vinculos as $vinculo){
            $vinculos_str .= $vinculo['NRVINCULOM'].",";
        }
        $vinculos_str = substr_replace($vinculos_str, '', -1); // remove ultima ','
        
        $filtro = array(
            'P_COMPETENCIA'=> $this->dtCompetencia,
            'P_NRORG'      => $this->nrorg,
            'P_ANOREF'     => $this->anoReferencia,
            'P_ANOCALEN'   => $this->anoCalendario,
            'P_VINCULO'    => $vinculos_str,
            'P_ESTRUTURARESP' => $this->nrEstrResponsa
        );
        
        $caminho = $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'informe_rendimentos', null, null);
        return $caminho; 
    }
    
    public function geraRelatorioAnalitico() {
        $params = array(
        "nrorg"             => $this->nrorg,
        "inicio"            => $this->dsInicio,
        "fim"               => $this->dsFim,
        "dtcompetencia"     => $this->dtCompetencia,
        "querypart"         => $this->queryPart
        );
        
        $vinculos = $this->queryDirf->get_nrvinculos($params,$this->queryPart);
        $vinculos_str;
        foreach($vinculos as $vinculo){
            $vinculos_str .= $vinculo['NRVINCULOM'].",";
        }
        $vinculos_str = substr_replace($vinculos_str, '', -1); // remove ultima ','
        
        $filtro = array(
            'P_COMPETENCIA'   => $this->dtCompetencia,
            'P_NRORG'         => $this->nrorg,
            'P_VINCULO'       => null/*$vinculos_str*/,  
            'P_ESTRUTURARESP' => $this->nrEstrResponsa
        );
        
        $caminho = $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'analitico_dirf', null, null);
        return $caminho;
        
    }
    
    
}