<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Model\Gestaopessoas\Entities\FpaTpcalcregra;

class PontoRegraService {
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function inserePontoRegraEventoProventoEventual($nrregrams, $parametros){
        //função que insere os pontos de regra  de tipo 'Após Cálculo Específico', um deles com tipo de calculo 'CALC_FOLHA' e outro  'CALC_RESCISAO' associados à regra de número passado
        //como parametro
        try{
            foreach($nrregrams as $nrregram){
                $dtInclusao = DateUtil::getDataAtual();
                $cdOperadorInclusao = $this->hcmParameters->getCdoperador();
                $nmtpcalcregra1 = "PONTO REGRA CALC_FOLHA " . $nrregram;
                $nmtpcalcregra2 = "PONTO REGRA CALC_RESCISAO " . $nrregram;
                $nrtpcalcregra1 = 1;
                $nrtpcalcregra2 = 6;
                
                $dadosPontoRegra = array('NRPTREGRA' => 70, 'NRSEQUENCIA' => 1, 'NRREGRAM' => $nrregram);
                $pontoRegra1 = array_merge($dadosPontoRegra, array('NRTIPOCALCULO' => $nrtpcalcregra1, 'NMTPCALCREGRA' => $nmtpcalcregra1));
                $pontoRegra2 = array_merge($dadosPontoRegra, array('NRTIPOCALCULO' => $nrtpcalcregra2, 'NMTPCALCREGRA' => $nmtpcalcregra2));
                
                $this->salvaPontoRegra(array($pontoRegra1, $pontoRegra2));
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function salvaPontoRegra($pontosRegra){
        try{
            foreach($pontosRegra as $ponto){
                $this->setPontoRegra($ponto);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function setPontoRegra($pontoRegra){
        
        $dtAtual = DateUtil::getDataAtual();
        $cdOperadorInclusao = $this->hcmParameters->getCdoperador();
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        try{
            $fpatpcalcregra = new FpaTpcalcregra();
                
            $this->entityManager->beginTransaction();
            $fpatpcalcregra->setCdoperinclusao(array_key_exists('CDOPERINCLUSAO', $pontoRegra) ? $pontoRegra['CDOPERINCLUSAO'] : $cdOperadorInclusao);
            $fpatpcalcregra->setCdoperultatu(array_key_exists('CDOPERULTATU', $pontoRegra) ? $pontoRegra['CDOPERULTATU'] : (array_key_exists('CDOPERINCLUSAO', $pontoRegra) ? $pontoRegra['CDOPERINCLUSAO'] : $cdOperadorInclusao));
            $fpatpcalcregra->setDtinclusao(array_key_exists('DTINCLUSAO', $pontoRegra) ? $pontoRegra['DTINCLUSAO'] : $dtAtual);
            $fpatpcalcregra->setDtultatu(array_key_exists('DTULTATU', $pontoRegra) ? $pontoRegra['DTULTATU'] : (array_key_exists('DTINCLUSAO', $pontoRegra) ? $pontoRegra['DTINCLUSAO'] : $dtAtual));
            $fpatpcalcregra->setNmtpcalcregra($pontoRegra['NMTPCALCREGRA']);
            $fpatpcalcregra->setNrorg(array_key_exists('NRORG', $pontoRegra) ? $pontoRegra['NRORG'] : $nrorg);
            $fpatpcalcregra->setNrorginclusao(array_key_exists('NRORGINCLUSAO', $pontoRegra) ? $pontoRegra['NRORGINCLUSAO'] : (array_key_exists('NRORG', $pontoRegra) ? $pontoRegra['NRORG'] : $nrorg));
            $fpatpcalcregra->setNrorgultatu(array_key_exists('NRORGULTATU', $pontoRegra) ? $pontoRegra['NRORGULTATU'] : (array_key_exists('NRORG', $pontoRegra) ? $pontoRegra['NRORG'] : $nrorg));
            $fpatpcalcregra->setNrptregra($pontoRegra['NRPTREGRA']);
            $fpatpcalcregra->setNrregram($pontoRegra['NRREGRAM']);
            $fpatpcalcregra->setNrsequencia($pontoRegra['NRSEQUENCIA']);
            $fpatpcalcregra->setNrtipocalculo($pontoRegra['NRTIPOCALCULO']);
            $fpatpcalcregra->setNrtpcalcregra(array_key_exists('NRTPCALCREGRA', $pontoRegra) ? $pontoRegra['NRTPCALCREGRA'] : \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_TPCALCREGRA', 1, 12, true));
            
            $this->entityManager->persist($fpatpcalcregra);
            $this->entityManager->flush();
            $this->entityManager->commit();   
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
}