<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Geral\Services\HierarquiaGridService;
use HCM\Util\Messages;
use HCM\Util\Util;

class HierarquiaGridController extends Crud {

    protected $dataSourceName = 'hierarquiaorg';
    protected $dataSourceNameGet = 'getHierarquiaGrid';
    protected $hierarquiaGridService;
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\Geral\Services\HierarquiaGridService $hierarquiaGridService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->hierarquiaGridService = $hierarquiaGridService;
        parent::__construct($dataSourceManager);
    }

    public function getHierarquiaGrid(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $buttonFloatingSearch = array();
        
        $nrestruturamrel = $nrestruturamsup = null;

        $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
        foreach($filters as $condition) {
            if($condition['columnName'] === 'NRESTRUTURAMREL'){
                $nrestruturamrel = str_replace("%", "", $condition['value']);            
            }elseif($condition['columnName'] === 'NRESTRUTURAMSUP'){
                $nrestruturamsup = str_replace("%", "", $condition['value']);            
            }elseif ($condition['operator'] === '=') {
                $conditions[] = $condition['columnName'] . " " . $condition['operator'] . " '" . $condition['value']."' ";
            } elseif ($condition['operator'] === 'BETWEEN') {
                $conditions[] = $condition['columnName'] . " " . $condition['operator'] . " '" . $condition['value'][0] . "' AND '" . $condition['value'][1] . "'";
            }elseif ($condition['operator'] === 'LIKE_ALL' || $condition['operator'] === 'LIKE_I'){
                $buttonFloatingSearch[] = $condition;
            }
        }
        
        $buttonFloatingSearch = Util::createQueryFiltered($buttonFloatingSearch);
        
        $dataSet = $this->hierarquiaGridService->getHierarquiaGrid($nrestruturamrel, $nrestruturamsup, $conditions, $buttonFloatingSearch);
        $response->addDataSet(new DataSet('getHierarquiaGrid', $dataSet));
    }

    public function getHierarquiaGridTree(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $buttonFloatingSearch = array();
        
        $nrestruturamrel = $nrestruturamsup = $nrtipohierarqorg = $nrtipoestrutsup = null;
        $filters = \HCM\Util\Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
        foreach($filters as $condition) {
            if($condition['columnName'] === 'NRESTRUTURAMREL'){
                $nrestruturamrel = str_replace("%", "", $condition['value']);            
            }elseif($condition['columnName'] === 'NRESTRUTURAMSUP'){
                $nrestruturamsup = str_replace("%", "", $condition['value']);            
            }elseif ($condition['columnName'] === 'NRTIPOHIERARQORG'){
                $nrtipohierarqorg = str_replace("%", "", $condition['value']);            
            }elseif ($condition['columnName'] === 'NRTIPOESTRUTSUP') {
                $nrtipoestrutsup = str_replace("%", "", $condition['value']);            
            }elseif ($condition['operator'] === '=') {
                $conditions[] = $condition['columnName'] . " " . $condition['operator'] . " '" . $condition['value']."' ";
            } elseif ($condition['operator'] === 'BETWEEN') {
                $conditions[] = $condition['columnName'] . " " . $condition['operator'] . " '" . $condition['value'][0] . "' AND '" . $condition['value'][1] . "'";
            }elseif ($condition['operator'] === 'LIKE_ALL' || $condition['operator'] === 'LIKE_I'){
                $buttonFloatingSearch[] = $condition;
            }
        }
        
        $buttonFloatingSearch = Util::createQueryFiltered($buttonFloatingSearch);
        
        $dataSet = $this->hierarquiaGridService->getHierarquiaGridTree($nrestruturamrel, $nrestruturamsup, $nrtipohierarqorg, $nrtipoestrutsup, $conditions, $buttonFloatingSearch);
        $response->addDataSet(new DataSet('getHierarquiaGridTree', $dataSet));
    }
    
    public function setHierarquiaGrid(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRHIERARQUIAORG', $row)) {
                    if ($row['NRHIERARQUIAORG'] === '') {
                        $row['NRHIERARQUIAORG'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'HIERARQUIAORG', 1, 12, true); 
                    }
                } else {
                    $row['NRHIERARQUIAORG'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'HIERARQUIAORG', 1, 12, true); 
                }
                if(!array_key_exists('NRESTRUTURAMSUP', $row)){
                    $row['NRESTRUTURAMSUP'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteHierarquiaGrid(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function validaCadastroHierarquiaGrid(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows()[0];
            $hierarquia = $this->hierarquiaGridService->retornaHierarquiaGridExistente($dataset);
            if($hierarquia){
                $Message = 'Essa hierarquia ainda está vigente ou a data de inicio converge com a última data de fim dessa hierarquia.';
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'E'));
            }else{
                $Message = 'Hierarquia válida.';
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'S'));   
            }
                    
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 