<?php
namespace Folha\Entities;

use HCM\Util\DateUtil;
use Geral\Util\Ambiente;
use Importacao\Logic\Integracao\ProjetoImportacao;

class MigDepvinculo extends Base\MigDepvinculo
{

    /**
     * @param $nrdepvinculo
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setNrdepvinculo ($nrdepvinculo) {
        $nrdepvinculo = \Importacao\Logic\Validacoes::verificaChaveDuplicada('DEPVINCULO', 'NRDEPVINCULO', ProjetoImportacao::getNrorg(), $nrdepvinculo);
        $this->nrdepvinculo = $nrdepvinculo;
    }

    /**
     * @param $nrvinculom
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setNrvinculom ($nrvinculom) {
        $nrvinculom = \Importacao\Logic\Validacoes::verificaChaveMae('GPE_VINCULOM', 'NRVINCULOM', ProjetoImportacao::getNrorg(), $nrvinculom);
        $this->nrvinculom = $nrvinculom;
    }

    /**
     * @param $nrpessoa
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setNrpessoa ($nrpessoa) {
        $nrpessoa = \Importacao\Logic\Validacoes::verificaChaveMae('GPE_PESSOA', 'NRPESSOA', ProjetoImportacao::getNrorg(), $nrpessoa);
        $this->nrpessoa = $nrpessoa;
    }

    /**
     * @param $nrtipodepende
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setNrtipodepende ($nrtipodepende) {        
        if (!empty($nrtipodepende)) {
            $nrorg = ProjetoImportacao::getNrorgPadrao();
            $tipodepende = \Folha\Entities\FpaTipodepende::findOneByNrtipodependeAndNrorg($nrtipodepende, $nrorg);

            if (is_object($tipodepende)) {
                $this->nrtipodepende = $nrtipodepende;
            } else {
                throw new \Exception("O código de Tipo de Dependência '$nrtipodepende' não foi encontrado.");
            }
        } else {
            throw new \Exception("O código de Tipo de Dependência '$nrtipodepende' não foi encontrado.");
        }
    }

    /**
     * @param $dtinidepende
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtinidepende ($dtinidepende) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinidepende)) {
            if (!empty($dtinidepende)) {
                throw new \Exception("Início Dependência: Data '$dtinidepende' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtinidepende instanceof \DateTime)) {
            $dtinidepende = DateUtil::getDataDeString($dtinidepende);
        }

        $this->dtinidepende = $dtinidepende;
    }

    /**
     * @param $dtfindepende
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtfindepende ($dtfindepende) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtfindepende)) {
            if (!empty($dtfindepende)) {
                throw new \Exception("Fim Dependência: Data '$dtfindepende' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtfindepende instanceof \DateTime)) {
            $dtfindepende = DateUtil::getDataDeString($dtfindepende);
        }

        $this->dtfindepende = $dtfindepende;
    }

    /**
     * @param $dtultatu
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtultatu ($dtultatu) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatu)) {
            throw new \Exception("Data '$dtultatu' com formato inválido ou em branco.");
        }

        if (!($dtultatu instanceof \DateTime)) {
            $dtultatu = DateUtil::getDataDeString($dtultatu);
        }

        $this->dtultatu = $dtultatu;
    }

    /**
     * @param $dtvercontrole
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtvercontrole ($dtvercontrole) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtvercontrole)) {
            if (!empty($dtvercontrole)) {
                throw new \Exception("Dt. Controle: Data '$dtvercontrole' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtvercontrole instanceof \DateTime)) {
            $dtvercontrole = DateUtil::getDataDeString($dtvercontrole);
        }

        $this->dtvercontrole = $dtvercontrole;
    }

    /**
     * @param $dtinclusao
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtinclusao ($dtinclusao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinclusao)) {
            throw new \Exception("Data '$dtinclusao' com formato inválido ou em branco.");
        }

        if (!($dtinclusao instanceof \DateTime)) {
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);
        }

        $this->dtinclusao = $dtinclusao;
    }

    /**
     * @param $dtinidependexc
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtinidependexc ($dtinidependexc) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinidependexc)) {
            if (!empty($dtinidependexc)) {
                throw new \Exception("Início Dependência: Data '$dtinidependexc' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtinidependexc instanceof \DateTime)) {
            $dtinidependexc = DateUtil::getDataDeString($dtinidependexc);
        }

        $this->dtinidependexc = $dtinidependexc;
    }

    /**
     * @param $dtfindependexc
     * @throws \Exception
     * @return \Folha\Entities\MigDepvinculo
     */
    public function setDtfindependexc ($dtfindependexc) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtfindependexc)) {
            if (!empty($dtfindependexc)) {
                throw new \Exception("Fim Dependência: Data '$dtfindependexc' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtfindependexc instanceof \DateTime)) {
            $dtfindependexc = DateUtil::getDataDeString($dtfindependexc);
        }

        $this->dtfindependexc = $dtfindependexc;
    }

}